Type: Package
Package: dplyr
Title: A Grammar of Data Manipulation
Version: 1.2.1
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Romain", "François", role = "aut",
           comment = c(ORCID = "0000-0002-2444-4226")),
    person("Lionel", "Henry", role = "aut"),
    person("Kirill", "Müller", role = "aut",
           comment = c(ORCID = "0000-0002-1416-3412")),
    person("Davis", "Vaughan", , "davis@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4777-038X")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: A fast, consistent tool for working with data frame like
    objects, both in memory and out of memory.
License: MIT + file LICENSE
URL: https://dplyr.tidyverse.org, https://github.com/tidyverse/dplyr
BugReports: https://github.com/tidyverse/dplyr/issues
Depends: R (>= 4.1.0)
Imports: cli (>= 3.6.2), generics, glue (>= 1.3.2), lifecycle (>=
        1.0.5), magrittr (>= 1.5), methods, pillar (>= 1.9.0), R6,
        rlang (>= 1.1.7), tibble (>= 3.2.0), tidyselect (>= 1.2.0),
        utils, vctrs (>= 0.7.1)
Suggests: broom, covr, DBI, dbplyr (>= 2.2.1), ggplot2, knitr, Lahman,
        lobstr, nycflights13, purrr, rmarkdown, RSQLite, stringi (>=
        1.7.6), testthat (>= 3.1.5), tidyr (>= 1.3.0), withr
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-02 19:51:05 UTC; hadleywickham
Author: Hadley Wickham [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Romain François [aut] (ORCID: <https://orcid.org/0000-0002-2444-4226>),
  Lionel Henry [aut],
  Kirill Müller [aut] (ORCID: <https://orcid.org/0000-0002-1416-3412>),
  Davis Vaughan [aut] (ORCID: <https://orcid.org/0000-0003-4777-038X>),
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-04-03 07:30:08 UTC
