test_that("generate informative errors", {
  expect_snapshot(error = TRUE, {
    add_count_()
    add_tally_()
    arrange_()
    count_()
    distinct_()
    do_()
    filter_()
    funs_()
    group_by_()
    group_indices_()
    mutate_()
    tally_()
    transmute_()
    rename_()
    select_()
    slice_()
    summarise_()
    summarize_()
  })
})
