# generate informative errors

    Code
      id()
    Condition
      Error:
      ! `id()` was deprecated in dplyr 0.5.0 and is now defunct.
      i Please use `vctrs::vec_group_id()` instead.
    Code
      failwith()
    Condition
      Error:
      ! `failwith()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `purrr::possibly()` instead.
    Code
      select_vars()
    Condition
      Error:
      ! `select_vars()` was deprecated in dplyr 0.8.4 and is now defunct.
      i Please use `tidyselect::vars_select()` instead.
    Code
      rename_vars()
    Condition
      Error:
      ! `rename_vars()` was deprecated in dplyr 0.8.4 and is now defunct.
      i Please use `tidyselect::vars_rename()` instead.
    Code
      select_var()
    Condition
      Error:
      ! `select_var()` was deprecated in dplyr 0.8.4 and is now defunct.
      i Please use `tidyselect::vars_pull()` instead.
    Code
      current_vars()
    Condition
      Error:
      ! `current_vars()` was deprecated in dplyr 0.8.4 and is now defunct.
      i Please use `tidyselect::peek_vars()` instead.
    Code
      bench_tbls()
    Condition
      Error:
      ! `bench_tbls()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      compare_tbls()
    Condition
      Error:
      ! `compare_tbls()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      compare_tbls2()
    Condition
      Error:
      ! `compare_tbls2()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      eval_tbls()
    Condition
      Error:
      ! `eval_tbls()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      eval_tbls2()
    Condition
      Error:
      ! `eval_tbls2()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      location()
    Condition
      Error:
      ! `location()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `lobstr::ref()` instead.
    Code
      changes()
    Condition
      Error:
      ! `changes()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `lobstr::ref()` instead.

