# generate informative errors

    Code
      combine()
    Condition
      Error:
      ! `combine()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `vctrs::vec_c()` instead.
    Code
      src_mysql()
    Condition
      Error:
      ! `src_mysql()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `tbl()` directly with a database connection
    Code
      src_postgres()
    Condition
      Error:
      ! `src_postgres()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `tbl()` directly with a database connection
    Code
      src_sqlite()
    Condition
      Error:
      ! `src_sqlite()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `tbl()` directly with a database connection
    Code
      src_local()
    Condition
      Error:
      ! `src_local()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      src_df()
    Condition
      Error:
      ! `src_df()` was deprecated in dplyr 1.0.0 and is now defunct.
    Code
      tbl_df()
    Condition
      Error:
      ! `tbl_df()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `tibble::as_tibble()` instead.
    Code
      as.tbl()
    Condition
      Error:
      ! `as.tbl()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `tibble::as_tibble()` instead.
    Code
      add_rownames()
    Condition
      Error:
      ! `add_rownames()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Please use `tibble::rownames_to_column()` instead.

