# generate informative errors

    Code
      add_count_()
    Condition
      Error:
      ! `add_count_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `add_count()` instead.
      i See vignette('programming') for more help
    Code
      add_tally_()
    Condition
      Error:
      ! `add_tally_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `add_tally()` instead.
      i See vignette('programming') for more help
    Code
      arrange_()
    Condition
      Error:
      ! `arrange_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `arrange()` instead.
      i See vignette('programming') for more help
    Code
      count_()
    Condition
      Error:
      ! `count_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `count()` instead.
      i See vignette('programming') for more help
    Code
      distinct_()
    Condition
      Error:
      ! `distinct_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `distinct()` instead.
      i See vignette('programming') for more help
    Code
      do_()
    Condition
      Error:
      ! `do_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `do()` instead.
      i See vignette('programming') for more help
    Code
      filter_()
    Condition
      Error:
      ! `filter_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `filter()` instead.
      i See vignette('programming') for more help
    Code
      funs_()
    Condition
      Error:
      ! `funs_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `funs()` instead.
      i See vignette('programming') for more help
    Code
      group_by_()
    Condition
      Error:
      ! `group_by_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `group_by()` instead.
      i See vignette('programming') for more help
    Code
      group_indices_()
    Condition
      Error:
      ! `group_indices_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `group_indices()` instead.
    Code
      mutate_()
    Condition
      Error:
      ! `mutate_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `mutate()` instead.
      i See vignette('programming') for more help
    Code
      tally_()
    Condition
      Error:
      ! `tally_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `tally()` instead.
      i See vignette('programming') for more help
    Code
      transmute_()
    Condition
      Error:
      ! `transmute_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `transmute()` instead.
      i See vignette('programming') for more help
    Code
      rename_()
    Condition
      Error:
      ! `rename_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `rename()` instead.
    Code
      select_()
    Condition
      Error:
      ! `select_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `select()` instead.
    Code
      slice_()
    Condition
      Error:
      ! `slice_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `slice()` instead.
    Code
      summarise_()
    Condition
      Error:
      ! `summarise_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `summarise()` instead.
    Code
      summarize_()
    Condition
      Error:
      ! `summarise_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `summarise()` instead.

