# errors cleanly on non-vectors

    Code
      desc(mean)
    Condition
      Error in `desc()`:
      ! `x` must be a vector, not a function.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

