# generate informative errors

    Code
      summarise_each()
    Condition
      Error:
      ! `summarise_each()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `across()` instead.
    Code
      summarise_each_()
    Condition
      Error:
      ! `summarise_each_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `across()` instead.
    Code
      mutate_each()
    Condition
      Error:
      ! `mutate_each()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `across()` instead.
    Code
      mutate_each_()
    Condition
      Error:
      ! `mutate_each_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `across()` instead.
    Code
      summarize_each()
    Condition
      Error:
      ! `summarise_each()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `across()` instead.
    Code
      summarize_each_()
    Condition
      Error:
      ! `summarise_each_()` was deprecated in dplyr 0.7.0 and is now defunct.
      i Please use `across()` instead.

