# group_cols(vars =) is defunct

    Code
      group_cols("a")
    Condition
      Error:
      ! The `vars` argument of `group_cols()` was deprecated in dplyr 1.0.0 and is now defunct.
      i Use `data` with entire dataframe instead

