% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/join-common-by.R
\name{common_by}
\alias{common_by}
\title{Extract out common by variables}
\usage{
common_by(by = NULL, x, y)
}
\description{
Extract out common by variables
}
\keyword{internal}
